@@ -26,8 +26,8 @@
 #include "xml-cache.h"
 
 
-#include <gnome-xml/tree.h>
-#include <gnome-xml/parser.h>
+#include <libxml/tree.h>
+#include <libxml/parser.h>
 
 #include <stdio.h>
 #include <time.h>
@@ -169,7 +169,7 @@ static GConfMetaInfo*query_metainfo  (GConfSource* source,
 
 static void          set_value       (GConfSource* source,
                                       const gchar* key,
-                                      GConfValue* value,
+                                      const GConfValue* value,
                                       GError** err);
 
 static GSList*       all_entries    (GConfSource* source,
@@ -429,7 +429,7 @@ query_value (GConfSource* source,
      problem, since some errors may be added that need reporting. */
   if (error != NULL)
     {
-      gconf_log(GCL_WARNING, error->message);
+      gconf_log(GCL_WARNING, "%s", error->message);
       g_error_free(error);
       error = NULL;
     }
@@ -449,7 +449,7 @@ query_value (GConfSource* source,
       /* perhaps we should be reporting this error... */
       if (error != NULL)
         {
-          gconf_log(GCL_WARNING, error->message);
+          gconf_log(GCL_WARNING, "%s", error->message);
           g_error_free(error);
           error = NULL;
         }
@@ -485,7 +485,7 @@ query_metainfo  (GConfSource* source, const gchar* key,
 }
 
 static void          
-set_value (GConfSource* source, const gchar* key, GConfValue* value,
+set_value (GConfSource* source, const gchar* key, const GConfValue* value,
            GError** err)
 {
   XMLSource* xs = (XMLSource*)source;
@@ -679,6 +679,9 @@ g_module_check_init (GModule *module)
 {
   gconf_log(GCL_DEBUG, _("Initializing XML backend module"));
 
+  LIBXML_TEST_VERSION;
+  xmlKeepBlanksDefault(1);
+
   return NULL;
 }
 
