@@ -32,7 +32,6 @@
 #include <sys/wait.h>
 #include <sys/time.h>
 #include <unistd.h>
-#include <liboaf/liboaf.h>
 
 /* Returns TRUE if there was an error, frees exception, sets err */
 static gboolean gconf_handle_corba_exception(CORBA_Environment* ev, GError** err);
@@ -153,8 +152,6 @@ static void       ctable_insert              (CnxnTable           *ct,
                                               GConfCnxn           *cnxn);
 static void       ctable_remove              (CnxnTable           *ct,
                                               GConfCnxn           *cnxn);
-static void       ctable_remove_by_client_id (CnxnTable           *ct,
-                                              guint                client_id);
 static GSList*    ctable_remove_by_conf      (CnxnTable           *ct,
                                               GConfEngine         *conf);
 static GConfCnxn* ctable_lookup_by_client_id (CnxnTable           *ct,
@@ -217,8 +214,8 @@ gconf_engine_set_database (GConfEngine *conf,
   conf->database = db;
 
   if (engines_by_db == NULL)
-    engines_by_db = g_hash_table_new ((GHashFunc) g_CORBA_Object_hash,
-                                      (GCompareFunc) g_CORBA_Object_equal);
+    engines_by_db = g_hash_table_new ((GHashFunc) gconf_CORBA_Object_hash,
+                                      (GCompareFunc) gconf_CORBA_Object_equal);
   
   g_hash_table_insert (engines_by_db, conf->database, conf);  
 }
@@ -287,7 +284,7 @@ gconf_engine_connect (GConfEngine *conf,
       if (err)
         *err = gconf_error_new(GCONF_ERROR_BAD_ADDRESS,
                                _("Server couldn't resolve the address `%s'"),
-                               conf->address);
+                               conf->address ? conf->address : "default");
           
       return FALSE;
     }
@@ -974,7 +971,7 @@ gconf_engine_get_default_from_schema (GConfEngine* conf,
 
 gboolean
 gconf_engine_set (GConfEngine* conf, const gchar* key,
-                  GConfValue* value, GError** err)
+                  const GConfValue* value, GError** err)
 {
   ConfigValue* cv;
   CORBA_Environment ev;
@@ -994,6 +991,9 @@ gconf_engine_set (GConfEngine* conf, const gchar* key,
   if (!gconf_key_check(key, err))
     return FALSE;
 
+  if (!gconf_value_validate (value, err))
+    return FALSE;
+  
   if (gconf_engine_is_local(conf))
     {
       GError* error = NULL;
@@ -1355,7 +1355,11 @@ gconf_engine_all_entries(GConfEngine* conf, const gchar* dir, GError** err)
       pair->is_default = is_defaults->_buffer[i];
       pair->is_writable = is_writables->_buffer[i];
       if (schema_names)
-        pair->schema_name = g_strdup (schema_names->_buffer[i]);
+        {
+          /* empty string means no schema name */
+          if (*(schema_names->_buffer[i]) != '\0')
+            pair->schema_name = g_strdup (schema_names->_buffer[i]);
+        }
       
       pairs = g_slist_prepend(pairs, pair);
       
@@ -1838,20 +1842,18 @@ static ConfigServer   server = CORBA_OBJECT_NIL;
 
 /* errors in here should be GCONF_ERROR_NO_SERVER */
 static ConfigServer
-try_to_contact_server(gboolean start_if_not_found, GError** err)
+try_to_contact_server (gboolean start_if_not_found,
+                       GError **err)
 {
   CORBA_Environment ev;
-  OAF_ActivationFlags flags;
   
-  CORBA_exception_init(&ev);
+  /* Try to launch server */      
+  server = gconf_activate_server (start_if_not_found,
+                                  err);
+    
+  /* Try to ping server, by adding ourselves as a client */
+  CORBA_exception_init (&ev);   
 
-  flags = 0;
-  if (!start_if_not_found)
-    flags |= OAF_FLAG_EXISTING_ONLY;
-  
-  server = oaf_activate_from_id ("OAFAID:["IID"]", flags, NULL, &ev);
-
-  /* So try to ping server, by adding ourselves as a client */
   if (!CORBA_Object_is_nil (server, &ev))
     {
       ConfigServer_add_client (server,
@@ -1861,33 +1863,19 @@ try_to_contact_server(gboolean start_if_not_found, GError** err)
       if (ev._major != CORBA_NO_EXCEPTION)
 	{
 	  server = CORBA_OBJECT_NIL;
-	  if (err)
-	    *err = gconf_error_new(GCONF_ERROR_NO_SERVER,
-                                   _("Adding client to server's list failed, CORBA error: %s"),
-				   CORBA_exception_id(&ev));
+          g_set_error (err,
+                       GCONF_ERROR,
+                       GCONF_ERROR_NO_SERVER,
+                       _("Adding client to server's list failed, CORBA error: %s"),
+                       CORBA_exception_id (&ev));
 
           CORBA_exception_free(&ev);
 	}
     }
-  else
-    {
-      if (gconf_handle_oaf_exception(&ev, err))
-        {
-          /* Make the errno more specific */
-          if (err && *err)
-            (*err)->code = GCONF_ERROR_NO_SERVER;
-        }
-
-      if (err && *err == NULL)
-        *err = gconf_error_new(GCONF_ERROR_NO_SERVER,
-                               _("Error contacting configuration server: OAF returned nil from oaf_activate_from_id() and did not set an exception explaining the problem. This is a bug in the OAF package; something went wrong in OAF, and no error was reported. This is not a bug in the GConf package. Do not report a GConf bug unless you have information indicating what went wrong with OAF that was caused by GConf."));
-    }
 
 #ifdef GCONF_ENABLE_DEBUG      
   if (server == CORBA_OBJECT_NIL && start_if_not_found)
-    {
-      g_return_val_if_fail(err == NULL || *err != NULL, server);
-    }
+    g_return_val_if_fail (err == NULL || *err != NULL, server);
 #endif
   
   return server;
@@ -2109,31 +2097,22 @@ drop_all_caches (PortableServer_Servant     _servant,
 static ConfigListener 
 gconf_get_config_listener(void)
 {
-  return listener;
-}
-
-static gboolean have_initted = FALSE;
-
-void
-gconf_preinit(gpointer app, gpointer mod_info)
-{
-}
-
-void
-gconf_postinit(gpointer app, gpointer mod_info)
-{
+  static ConfigListener listener = CORBA_OBJECT_NIL;
+  
   if (listener == CORBA_OBJECT_NIL)
     {
       CORBA_Environment ev;
-      PortableServer_ObjectId objid = {0, sizeof("ConfigListener"), "ConfigListener"};
+      PortableServer_ObjectId* objid;
       PortableServer_POA poa;
 
-      CORBA_exception_init(&ev);
-      POA_ConfigListener__init(&poa_listener_servant, &ev);
+      CORBA_exception_init (&ev);
+      POA_ConfigListener__init (&poa_listener_servant, &ev);
       
       g_assert (ev._major == CORBA_NO_EXCEPTION);
 
-      poa = (PortableServer_POA)CORBA_ORB_resolve_initial_references(oaf_orb_get(), "RootPOA", &ev);
+      poa =
+        (PortableServer_POA) CORBA_ORB_resolve_initial_references (gconf_orb_get (),
+                                                                   "RootPOA", &ev);
 
       g_assert (ev._major == CORBA_NO_EXCEPTION);
 
@@ -2141,8 +2120,7 @@ gconf_postinit(gpointer app, gpointer mod_info)
 
       g_assert (ev._major == CORBA_NO_EXCEPTION);
 
-      PortableServer_POA_activate_object_with_id(poa,
-                                                 &objid, &poa_listener_servant, &ev);
+      objid = PortableServer_POA_activate_object (poa, &poa_listener_servant, &ev);
 
       g_assert (ev._major == CORBA_NO_EXCEPTION);
       
@@ -2153,49 +2131,33 @@ gconf_postinit(gpointer app, gpointer mod_info)
       g_assert (listener != CORBA_OBJECT_NIL);
       g_assert (ev._major == CORBA_NO_EXCEPTION);
     }
-
-  have_initted = TRUE;
+  
+  return listener;
+}
+     
+void
+gconf_preinit (gpointer app, gpointer mod_info)
+{
+  /* Deprecated */
 }
 
+void
+gconf_postinit (gpointer app, gpointer mod_info)
+{
+  /* Deprecated */
+}
+
+/* All deprecated */
 const char gconf_version[] = VERSION;
 
 struct poptOption gconf_options[] = {
   {NULL}
 };
 
+/* Also deprecated */
 gboolean     
-gconf_init           (int argc, char **argv, GError** err)
+gconf_init (int argc, char **argv, GError** err)
 {
-  CORBA_ORB orb = CORBA_OBJECT_NIL;
-
-  if (have_initted)
-    {
-      g_warning("Attempt to init GConf a second time");
-      return FALSE;
-    }
-
-  gconf_preinit(NULL, NULL);
-
-  if (!oaf_is_initialized())
-    {
-      orb = oaf_init(argc, argv);
-    }
-  else
-    {
-      orb = oaf_orb_get();
-    }
-      
-  gconf_postinit(NULL, NULL);
-
-  if(!have_initted)
-    {
-      if (err == NULL)
-        {
-          fprintf(stderr, _("Failed to init GConf, exiting\n"));
-          exit (1);
-        }
-      return FALSE;
-    }
   
   return TRUE;
 }
@@ -2203,7 +2165,7 @@ gconf_init           (int argc, char **argv, GError** err)
 gboolean
 gconf_is_initialized (void)
 {
-  return have_initted;
+  return TRUE;
 }
 
 /* 
@@ -2216,6 +2178,8 @@ gconf_is_initialized (void)
 /* Key/dir validity is exactly the same, except that '/' must be a dir, 
    but we are sort of ignoring that for now. */
 
+/* Also, keys can contain only ASCII */
+
 static const gchar invalid_chars[] = " \t\r\n\"$&<>,+=#!()'|{}[]?~`;%\\";
 
 gboolean     
@@ -2267,7 +2231,15 @@ gconf_valid_key      (const gchar* key, gchar** why_invalid)
           const gchar* inv = invalid_chars;
 
           just_saw_slash = FALSE;
-
+          
+          if (((unsigned char)*s) > 127)
+            {
+              if (why_invalid != NULL)
+                *why_invalid = g_strdup_printf (_("'%c' is not an ASCII character, so isn't allowed in key names"),
+                                                *s);
+              return FALSE;
+            }
+          
           while (*inv)
             {
               if (*inv == *s)
@@ -2294,6 +2266,122 @@ gconf_valid_key      (const gchar* key, gchar** why_invalid)
     return TRUE;
 }
 
+/**
+ * gconf_escape_key:
+ * @arbitrary_text: some text in any encoding or format
+ * @len: length of @arbitrary_text in bytes, or -1 if @arbitrary_text is nul-terminated
+ * 
+ * Escape @arbitrary_text such that it's a valid key element (i.e. one
+ * part of the key path). The escaped key won't pass gconf_valid_key()
+ * because it isn't a whole key (i.e. it doesn't have a preceding
+ * slash), but prepending a slash to the escaped text should always
+ * result in a valid key.
+ * 
+ * Return value: a nul-terminated valid GConf key
+ **/
+char*
+gconf_escape_key (const char *arbitrary_text,
+                  int         len)
+{
+  const char *p;
+  const char *end;
+  GString *retval;
+
+  g_return_val_if_fail (arbitrary_text != NULL, NULL);
+  
+  /* Nearly all characters we would normally use for escaping aren't allowed in key
+   * names, so we use @ for that.
+   *
+   * Invalid chars and @ itself are escaped as @xxx@ where xxx is the
+   * Latin-1 value in decimal
+   */
+
+  if (len < 0)
+    len = strlen (arbitrary_text);
+
+  retval = g_string_new ("");
+
+  p = arbitrary_text;
+  end = arbitrary_text + len;
+  while (p != end)
+    {
+      if (*p == '/' || *p == '.' || *p == '@' || ((guchar) *p) > 127 ||
+          strchr (invalid_chars, *p))
+        {
+          g_string_append_c (retval, '@');
+          g_string_append_printf (retval, "%u", (unsigned int) *p);
+          g_string_append_c (retval, '@');
+        }
+      else
+        g_string_append_c (retval, *p);
+      
+      ++p;
+    }
+
+  return g_string_free (retval, FALSE);
+}
+
+/**
+ * gconf_unescape_key:
+ * @escaped_key: a key created with gconf_escape_key()
+ * @len: length of @escaped_key in bytes, or -1 if @escaped_key is nul-terminated
+ * 
+ * Converts a string escaped with gconf_escape_key() back into its original
+ * form.
+ * 
+ * Return value: the original string that was escaped to create @escaped_key
+ **/
+char*
+gconf_unescape_key (const char *escaped_key,
+                    int         len)
+{
+  const char *p;
+  const char *end;
+  const char *start_seq;
+  GString *retval;
+
+  g_return_val_if_fail (escaped_key != NULL, NULL);
+  
+  if (len < 0)
+    len = strlen (escaped_key);
+
+  retval = g_string_new ("");
+
+  p = escaped_key;
+  end = escaped_key + len;
+  start_seq = NULL;
+  while (p != end)
+    {
+      if (start_seq)
+        {
+          if (*p == '@')
+            {
+              /* *p is the @ that ends a seq */
+              char *end;
+              guchar val;
+              
+              val = strtoul (start_seq, &end, 10);
+              if (start_seq != end)
+                g_string_append_c (retval, val);
+              
+              start_seq = NULL;
+            }
+        }
+      else
+        {
+          if (*p == '@')
+            start_seq = p + 1;
+          else
+            g_string_append_c (retval, *p);
+        }
+
+      ++p;
+    }
+
+  return g_string_free (retval, FALSE);
+}
+
+
 gboolean
 gconf_key_is_below   (const gchar* above, const gchar* below)
 {
@@ -2420,18 +2508,6 @@ ctable_remove(CnxnTable* ct, GConfCnxn* cnxn)
   g_hash_table_remove (ct->client_ids, &cnxn->client_id);
 }
 
-static void       
-ctable_remove_by_client_id(CnxnTable* ct, guint client_id)
-{
-  GConfCnxn* cnxn;
-
-  cnxn = ctable_lookup_by_client_id (ct, client_id);
-
-  g_return_if_fail (cnxn != NULL);
-
-  ctable_remove (ct, cnxn);
-}
-
 struct RemoveData {
   GSList* removed;
   GConfEngine* conf;
@@ -2760,7 +2836,7 @@ gconf_engine_get_schema  (GConfEngine* conf, const gchar* key, GError** err)
 
 GSList*
 gconf_engine_get_list    (GConfEngine* conf, const gchar* key,
-                   GConfValueType list_type, GError** err)
+                          GConfValueType list_type, GError** err)
 {
   GConfValue* val;
 
@@ -2859,7 +2935,7 @@ error_checked_set(GConfEngine* conf, const gchar* key,
 
 gboolean
 gconf_engine_set_float   (GConfEngine* conf, const gchar* key,
-                   gdouble val, GError** err)
+                          gdouble val, GError** err)
 {
   GConfValue* gval;
 
@@ -2876,7 +2952,7 @@ gconf_engine_set_float   (GConfEngine* conf, const gchar* key,
 
 gboolean
 gconf_engine_set_int     (GConfEngine* conf, const gchar* key,
-                   gint val, GError** err)
+                          gint val, GError** err)
 {
   GConfValue* gval;
 
@@ -2893,14 +2969,16 @@ gconf_engine_set_int     (GConfEngine* conf, const gchar* key,
 
 gboolean
 gconf_engine_set_string  (GConfEngine* conf, const gchar* key,
-                    const gchar* val, GError** err)
+                          const gchar* val, GError** err)
 {
   GConfValue* gval;
 
-  g_return_val_if_fail(val != NULL, FALSE);
-  g_return_val_if_fail(conf != NULL, FALSE);
-  g_return_val_if_fail(key != NULL, FALSE);
-  g_return_val_if_fail(err == NULL || *err == NULL, FALSE);
+  g_return_val_if_fail (val != NULL, FALSE);
+  g_return_val_if_fail (conf != NULL, FALSE);
+  g_return_val_if_fail (key != NULL, FALSE);
+  g_return_val_if_fail (err == NULL || *err == NULL, FALSE);
+  
+  g_return_val_if_fail (g_utf8_validate (val, -1, NULL), FALSE);
   
   gval = gconf_value_new(GCONF_VALUE_STRING);
 
@@ -2911,7 +2989,7 @@ gconf_engine_set_string  (GConfEngine* conf, const gchar* key,
 
 gboolean
 gconf_engine_set_bool    (GConfEngine* conf, const gchar* key,
-                   gboolean val, GError** err)
+                          gboolean val, GError** err)
 {
   GConfValue* gval;
 
@@ -2928,7 +3006,7 @@ gconf_engine_set_bool    (GConfEngine* conf, const gchar* key,
 
 gboolean
 gconf_engine_set_schema  (GConfEngine* conf, const gchar* key,
-                    GConfSchema* val, GError** err)
+                          const GConfSchema* val, GError** err)
 {
   GConfValue* gval;
 
@@ -2946,11 +3024,12 @@ gconf_engine_set_schema  (GConfEngine* conf, const gchar* key,
 
 gboolean
 gconf_engine_set_list    (GConfEngine* conf, const gchar* key,
-                   GConfValueType list_type,
-                   GSList* list,
-                   GError** err)
+                          GConfValueType list_type,
+                          GSList* list,
+                          GError** err)
 {
   GConfValue* value_list;
+  GError *tmp_err = NULL;
   
   g_return_val_if_fail(conf != NULL, FALSE);
   g_return_val_if_fail(key != NULL, FALSE);
@@ -2959,7 +3038,13 @@ gconf_engine_set_list    (GConfEngine* conf, const gchar* key,
   g_return_val_if_fail(list_type != GCONF_VALUE_PAIR, FALSE);
   g_return_val_if_fail(err == NULL || *err == NULL, FALSE);
 
-  value_list = gconf_value_list_from_primitive_list(list_type, list);
+  value_list = gconf_value_list_from_primitive_list(list_type, list, &tmp_err);
+
+  if (tmp_err)
+    {
+      g_propagate_error (err, tmp_err);
+      return FALSE;
+    }
   
   /* destroys the value_list */
   
@@ -2968,12 +3053,13 @@ gconf_engine_set_list    (GConfEngine* conf, const gchar* key,
 
 gboolean
 gconf_engine_set_pair    (GConfEngine* conf, const gchar* key,
-                   GConfValueType car_type, GConfValueType cdr_type,
-                   gconstpointer address_of_car,
-                   gconstpointer address_of_cdr,
-                   GError** err)
+                          GConfValueType car_type, GConfValueType cdr_type,
+                          gconstpointer address_of_car,
+                          gconstpointer address_of_cdr,
+                          GError** err)
 {
   GConfValue* pair;
+  GError *tmp_err = NULL;
   
   g_return_val_if_fail(conf != NULL, FALSE);
   g_return_val_if_fail(key != NULL, FALSE);
@@ -2989,7 +3075,14 @@ gconf_engine_set_pair    (GConfEngine* conf, const gchar* key,
   
 
   pair = gconf_value_pair_from_primitive_pair(car_type, cdr_type,
-                                              address_of_car, address_of_cdr);
+                                              address_of_car, address_of_cdr,
+                                              &tmp_err);
+
+  if (tmp_err)
+    {
+      g_propagate_error (err, tmp_err);
+      return FALSE;
+    }  
   
   return error_checked_set(conf, key, pair, err);
 }
