@@ -21,6 +21,7 @@
 #include "gconf-internals.h"
 #include <stdio.h>
 #include <unistd.h>
+#include <string.h>
 #include <popt.h>
 #include <libxml/tree.h>
 #include <libxml/parser.h>
@@ -47,7 +48,7 @@ static char* short_desc = NULL;
 static char* long_desc = NULL;
 static char* owner = NULL;
 static char* schema_file = NULL;
-static char* config_source = NULL;
+static const char* config_source = NULL;
 static int use_local_source = FALSE;
 static int makefile_install_mode = FALSE;
 static int break_key_mode = FALSE;
@@ -369,8 +370,11 @@ main (int argc, char** argv)
   poptContext ctx;
   gint nextopt;
   GError* err = NULL;
+
+  LIBXML_TEST_VERSION;
+  xmlKeepBlanksDefault(1);
   
-  ctx = poptGetContext("gconftool", argc, argv, options, 0);
+  ctx = poptGetContext("gconftool", argc, (const char **) argv, options, 0);
 
   poptReadDefaultConfig(ctx, TRUE);
 
@@ -1723,7 +1727,7 @@ extract_global_info(xmlNodePtr node,
   xmlNodePtr iter;
   char* default_value = NULL;
       
-  iter = node->childs;
+  iter = node->xmlChildrenNode;
 
   while (iter != NULL)
     {
@@ -1928,7 +1932,7 @@ process_locale_info(xmlNodePtr node, SchemaInfo* info)
 
 
   /* Locale-specific info */
-  iter = node->childs;
+  iter = node->xmlChildrenNode;
   
   while (iter != NULL)
     {
@@ -2082,7 +2086,7 @@ process_schema(GConfEngine* conf, xmlNodePtr node)
       return 1;
     }
   
-  iter = node->childs;
+  iter = node->xmlChildrenNode;
 
   while (iter != NULL)
     {
@@ -2144,7 +2148,7 @@ process_schema_list(GConfEngine* conf, xmlNodePtr node)
 {
   xmlNodePtr iter;
 
-  iter = node->childs;
+  iter = node->xmlChildrenNode;
 
   while (iter != NULL)
     {
@@ -2178,14 +2182,14 @@ do_load_schema_file(GConfEngine* conf, const gchar* file)
       return 1;
     }
 
-  if (doc->root == NULL)
+  if (doc->xmlRootNode == NULL)
     {
       fprintf(stderr, _("Document `%s' is empty?\n"),
               file);
       return 1;
     }
 
-  iter = doc->root;
+  iter = doc->xmlRootNode;
   while (iter != NULL) 
     {
       if (iter->type == XML_ELEMENT_NODE)
@@ -2210,7 +2214,7 @@ do_load_schema_file(GConfEngine* conf, const gchar* file)
       return 1;
     }
 
-  iter = iter->childs;
+  iter = iter->xmlChildrenNode;
 
   while (iter != NULL)
     {
